import express from "express";

import {database} from "./database";
import {FirestoreClient, FirestoreCollection} from "./FirestoreClient";
import {WhereFilterOp} from "@google-cloud/firestore";

class HttpHandler {

  public app: express.Application
  constructor() {
    this.app = express()

    this.app.put('*', async (req, res) => {
        const firestoreClient = new FirestoreClient(database.getDb())
        let result
        let collection

        collection = firestoreClient.getCollection(req.body.collectionName, req.body.optionalFields)

        if (req.body.subCollection && req.body.documentId) {
            collection = collection.getSubCollection(req.body.documentId, req.body.subCollection)
        }


        if (req.body.updateIdInTransaction) {
            result = await firestoreClient.runTransaction(async transaction =>  {
                const document = await collection.getDocument(req.body.updateIdInTransaction as string, transaction)
                collection.setDocumentInTransaction(document.id, document, transaction)
            })
            return res.status(200).send(result)
        }


        if (req.body.methodName === "getQuery") {
            const q : {fieldPath: string, opStr: WhereFilterOp, value: any}[] = req.body.args[0]

            let firestoreQuery = collection.startQuery(q[0].fieldPath, q[0].opStr, q[0].value)
            for (let i = 1; i < q.length; i++) {
                firestoreQuery = firestoreQuery.where(q[i].fieldPath, q[i].opStr, q[i].value)
            }

            result = await collection.getQueryResult(firestoreQuery)
            return res.status(200).send(result)
        }
        result = await collection[req.body.methodName](...req.body.args)
        return res.status(200).send(result)
      })
  }


}

export const app = new HttpHandler().app

