"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirestoreUtil = exports.FirestoreCollection = exports.FirestoreClient = void 0;
const admin = __importStar(require("firebase-admin"));
const lodash_chunk_1 = __importDefault(require("lodash.chunk"));
const MAX_BATCH_SIZE = 500;
const MAX_WHERE_IN = 10;
class FirestoreClient {
    constructor(firestore) {
        this.firestore = firestore;
    }
    getCollection(collectionName, optionalFields = [], versioned = true) {
        return new FirestoreCollection(this.firestore.collection(collectionName), this.firestore, optionalFields, versioned);
    }
    runTransaction(updateFunction, transactionOptions) {
        return this.firestore.runTransaction(updateFunction, transactionOptions);
    }
}
exports.FirestoreClient = FirestoreClient;
class FirestoreCollection {
    constructor(collectionReference, firestore, optionalFields, versioned = true) {
        this.collectionReference = collectionReference;
        this.firestore = firestore;
        this.optionalFields = optionalFields;
        this.versioned = versioned;
    }
    async documentExists(id, transaction) {
        const docRef = typeof id === "string" ? this.collectionReference.doc(id) : id;
        const documentSnapshot = transaction ? await transaction.get(docRef) : await docRef.get();
        if (documentSnapshot === undefined || !documentSnapshot.exists) {
            return Promise.resolve(false);
        }
        return Promise.resolve(true);
    }
    async getDocument(id, transaction) {
        const docRef = typeof id === "string" ? this.collectionReference.doc(id) : id;
        const documentSnapshot = transaction ? await transaction.get(docRef) : await docRef.get();
        if (documentSnapshot === undefined || !documentSnapshot.exists) {
            return Promise.resolve(undefined);
        }
        return Promise.resolve(FirestoreUtil.decode(documentSnapshot));
    }
    getSubCollection(documentId, collectionName, optionalFields = [], versioned = true) {
        this.collectionReference.doc(documentId).collection(collectionName);
        return new FirestoreCollection(this.collectionReference.doc(documentId).collection(collectionName), this.firestore, optionalFields, versioned);
    }
    async setDocument(id, data) {
        this.deleteOptionalFields(data);
        if (this.versioned)
            this.applyVersionIncrement(FirestoreUtil.datesToTimestamps(data));
        const docRef = this.collectionReference.doc(id);
        return docRef.set(data, { merge: true });
    }
    async listDocuments() {
        const docRefs = await this.collectionReference.listDocuments();
        if (docRefs !== undefined) {
            return Promise.all(docRefs.map(async (doc) => FirestoreUtil.decode(await doc.get())));
        }
        return Promise.reject();
    }
    async addDocument(data) {
        if (this.versioned)
            this.applyVersionIncrement(data);
        const docRef = await this.collectionReference.add(FirestoreUtil.datesToTimestamps(data));
        if (docRef === undefined) {
            return Promise.reject();
        }
        return this.getDocument(docRef);
    }
    async updateDocument(id, updateData, transaction) {
        if (this.versioned)
            this.applyVersionIncrement(updateData);
        const docRef = this.collectionReference.doc(id);
        transaction ? await transaction.update(docRef, FirestoreUtil.datesToTimestamps(updateData)) : await docRef.update(updateData);
        return this.getDocument(id);
    }
    async deleteDocument(id) {
        return this.collectionReference.doc(id).delete();
    }
    setDocumentInTransaction(id, data, transaction) {
        this.deleteOptionalFields(data);
        if (this.versioned)
            this.applyVersionIncrement(data);
        let docRef;
        if (!id) {
            docRef = this.collectionReference.doc();
        }
        else {
            docRef = typeof id === "string" ? this.collectionReference.doc(id) : id;
        }
        return transaction.set(docRef, FirestoreUtil.datesToTimestamps(data), { merge: true });
    }
    async executeQuery(fieldPath, opStr, value) {
        const querySnapshot = await this.collectionReference.where(fieldPath, opStr, value).get();
        if (querySnapshot !== undefined) {
            return querySnapshot.docs.map(doc => FirestoreUtil.decode(doc));
        }
        return Promise.reject();
    }
    startQuery(fieldPath, opStr, value) {
        return this.collectionReference.where(fieldPath, opStr, value);
    }
    async getQueryResult(query) {
        const querySnapshot = await query.get();
        if (querySnapshot !== undefined) {
            return querySnapshot.docs.map(doc => FirestoreUtil.decode(doc));
        }
        return Promise.reject();
    }
    bulkSet(data) {
        Promise.all(lodash_chunk_1.default(data, MAX_BATCH_SIZE).map(async (dataRefs) => {
            const writeBatch = this.firestore.batch();
            (dataRefs).map(value => {
                this.deleteOptionalFields(value);
                if (this.versioned)
                    this.applyVersionIncrement(value);
                const docRef = value.id ? this.collectionReference.doc(value.id) : this.collectionReference.doc();
                writeBatch.set(docRef, FirestoreUtil.datesToTimestamps(value), { merge: true });
            });
            await writeBatch.commit();
        })).catch(reason => {
            return Promise.reject();
        });
    }
    async getDocumentsById(ids, transaction) {
        let result = [];
        await Promise.all(lodash_chunk_1.default(ids, MAX_WHERE_IN).map(async (idsChunk) => {
            const query = this.collectionReference.where(admin.firestore.FieldPath.documentId(), "in", idsChunk);
            const queryResult = await (transaction ? transaction.get(query) : query.get());
            result = result.concat(queryResult.docs.map(value => FirestoreUtil.decode(value)));
        }));
        return result;
    }
    applyVersionIncrement(data) {
        data.version = admin.firestore.FieldValue.increment(1);
    }
    deleteOptionalFields(data) {
        this.optionalFields
            .filter(prop => data[prop] === undefined)
            .map(value => data[value] = admin.firestore.FieldValue.delete());
    }
}
exports.FirestoreCollection = FirestoreCollection;
class FirestoreUtil {
    static decode(documentSnapshot) {
        return Object.assign(Object.assign({}, this.timestampsToDates(documentSnapshot.data())), { created: documentSnapshot.createTime.toDate(), lastModified: documentSnapshot.updateTime.toDate(), id: documentSnapshot.id });
    }
    static timestampsToDates(data) {
        Object.keys(data).forEach(key => {
            if (data[key] && typeof data[key] === 'object' && (data[key]["_seconds"] !== undefined && data[key]["_nanoseconds"] !== undefined))
                data[key] = data[key].toDate();
            else if (data[key] && typeof data[key] === 'object') {
                this.timestampsToDates(data[key]);
            }
        });
        return data;
    }
    static datesToTimestamps(data) {
        Object.keys(data).forEach(key => {
            if (data[key] && typeof data[key] === 'object' && data[key] instanceof Date)
                data[key] = admin.firestore.Timestamp.fromDate(data[key]);
            else if (data[key] && typeof data[key] === 'object') {
                this.datesToTimestamps(data[key]);
            }
            else if (data[key] && typeof data[key] === 'string') {
                const parsedDate = this.parseIsoDate(data[key]);
                if (parsedDate != undefined) {
                    data[key] = admin.firestore.Timestamp.fromDate(parsedDate);
                }
            }
        });
        return data;
    }
    static parseIsoDate(input) {
        const iso = /^(\d{4})(?:-?W(\d+)(?:-?(\d+)D?)?|(?:-(\d+))?-(\d+))(?:[T ](\d+):(\d+)(?::(\d+)(?:\.(\d+))?)?)?(?:Z(-?\d*))?$/;
        const parts = input.match(iso);
        if (parts == null) {
            return undefined;
        }
        const year = Number(parts[1]);
        let months;
        let days;
        if (typeof parts[2] != "undefined") {
            const weeks = Number(parts[2]) - 1;
            days = Number(parts[3]);
            if (typeof days == "undefined") {
                days = 0;
            }
            days += weeks * 7;
            months = 0;
        }
        else {
            if (typeof parts[4] != "undefined") {
                months = Number(parts[4]) - 1;
            }
            else {
                months = 0;
            }
            days = Number(parts[5]);
        }
        let minutes;
        let hours;
        let seconds = 0;
        let milliseconds = 0;
        let fractional;
        if (typeof parts[6] != "undefined" &&
            typeof parts[7] != "undefined") {
            hours = Number(parts[6]);
            minutes = Number(parts[7]);
            if (typeof parts[8] != "undefined") {
                const seconds = Number(parts[8]);
                if (typeof parts[9] != "undefined") {
                    fractional = Number(parts[9]);
                    milliseconds = fractional / 100;
                }
                else {
                    milliseconds = 0;
                }
            }
            else {
                seconds = 0;
                milliseconds = 0;
            }
        }
        else {
            hours = 0;
            minutes = 0;
            seconds = 0;
            fractional = 0;
            milliseconds = 0;
        }
        if (typeof parts[10] != "undefined") {
            const localzone = -(new Date().getTimezoneOffset());
            const timezone = parts[10] * 60;
            minutes = Number(minutes) + (timezone - localzone);
        }
        return new Date(year, months, days, hours, minutes, seconds, milliseconds);
    }
}
exports.FirestoreUtil = FirestoreUtil;
//# sourceMappingURL=FirestoreClient.js.map