"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.app = void 0;
const express_1 = __importDefault(require("express"));
const database_1 = require("./database");
const FirestoreClient_1 = require("./FirestoreClient");
class HttpHandler {
    constructor() {
        this.app = express_1.default();
        this.app.put('*', async (req, res) => {
            const firestoreClient = new FirestoreClient_1.FirestoreClient(database_1.database.getDb());
            let result;
            let collection;
            collection = firestoreClient.getCollection(req.body.collectionName, req.body.optionalFields);
            if (req.body.subCollection && req.body.documentId) {
                collection = collection.getSubCollection(req.body.documentId, req.body.subCollection);
            }
            if (req.body.updateIdInTransaction) {
                result = await firestoreClient.runTransaction(async (transaction) => {
                    const document = await collection.getDocument(req.body.updateIdInTransaction, transaction);
                    collection.setDocumentInTransaction(document.id, document, transaction);
                });
                return res.status(200).send(result);
            }
            if (req.body.methodName === "getQuery") {
                const q = req.body.args[0];
                let firestoreQuery = collection.startQuery(q[0].fieldPath, q[0].opStr, q[0].value);
                for (let i = 1; i < q.length; i++) {
                    firestoreQuery = firestoreQuery.where(q[i].fieldPath, q[i].opStr, q[i].value);
                }
                result = await collection.getQueryResult(firestoreQuery);
                return res.status(200).send(result);
            }
            result = await collection[req.body.methodName](...req.body.args);
            return res.status(200).send(result);
        });
    }
}
exports.app = new HttpHandler().app;
//# sourceMappingURL=HttpHandler.js.map