import {FirestoreClient, FirestoreUtil} from "../FirestoreClient"
import * as admin from "firebase-admin";
import Timestamp = admin.firestore.Timestamp;
import {expect} from "@jest/globals";



const firestoreMock = { collection: jest.fn() }
const firestoreCollectionMock = {
    add: jest.fn(),
}
const firestoreDocRefMock = {
    get: jest.fn()
}

const firestoreClient = new FirestoreClient(firestoreMock as any)

afterEach(() => {
    jest.clearAllMocks(); // Restore the default sandbox here
});


describe('Unit Tests', () => {

    it('should create a collection reference', async() => {
        //given
        const collectionName = "test-collection"


        //when
        firestoreClient.getCollection(collectionName)

        //then
        expect(firestoreMock.collection).toBeCalledTimes(1)
        expect(firestoreMock.collection.mock.calls[0][0]).toBe(collectionName)

    });


    it('should transfrom dates', async() => {
        //given
        const date = '2020-04-10T08:50:00.000Z'

        const someObject = {
            someDate: new Date(date),
            someString: "someString"
        }

        //when
        const result = FirestoreUtil.datesToTimestamps(someObject)

        //then
        expect(result.someDate).toEqual({"_nanoseconds": 0, "_seconds": 1586508600})


        //and

        const retransfrom = FirestoreUtil.timestampsToDates(result)

        expect(retransfrom).toEqual(someObject)

        let d = new Date()
        d.setFullYear(2020)
        d.setUTCMonth(1)
        d.setUTCDate(1)
        d.setUTCHours(10)
        d.setUTCMinutes(1)
        d.setUTCSeconds(1)
        d.setUTCMilliseconds(100)
        const entityForCreation = {
            someOptionalDate: d}

        //when
        const result2 = FirestoreUtil.datesToTimestamps(entityForCreation)

        //then
        expect(result2.someOptionalDate).toEqual({
            "_nanoseconds": 100000000,
            "_seconds": 1580551261
        })


        //and

        const retransfrom2 = FirestoreUtil.timestampsToDates(result2)
        expect(retransfrom2).toEqual(entityForCreation)

    });
});
